/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */
#include <QDockWidget>

#include "main.h"
#include "window_list.h"
#include "help.h"
#include "table.h"
#include <QInputDialog>
#include <QFileDialog>
#include <QFileInfo>
#include <QDir>
#include <QMessageBox>
#include "generate_menu.h"
#include "mainwindow.h"
#include "windows_layout.h"



Main::Main(QObject * parent ):QObject (parent)
{
	
	OctaveConnection *oc = new OctaveConnection();
	main_window = new MainWindow(oc);
	
	work_space=main_window->work_space;
	
	window_list=NULL;
	
	terminal = new Terminal(work_space);
	terminal->work_space=work_space;
	connect(terminal, SIGNAL(widget_activated(BaseWidget *)), this, SLOT(widget_activated(BaseWidget *)));


	QString modeWorkArea = get_config("mode_work_area");	
	if ( modeWorkArea == "workspace")
	{
		//Show list of windows in workspace
		window_list = new WindowList(work_space,main_window->toolBarDocks);
		main_window->toolBarDocks->addWidget(window_list);
		window_list->setSizePolicy(QSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred)); 
		window_list->show();
		//Add terminal to workspace
		work_space->addWindow(terminal);
		terminal->show();
	}
	//add Terminal Dock 
	else if (modeWorkArea == "dock")
	{
		main_window->dockTerminal->setWidget(terminal);
	}
	else if (modeWorkArea == "sdi")
	{
		main_window->setCentralWidget(terminal);
	}
	
	connect(main_window->actionCompletionMatches, SIGNAL(triggered()), terminal, SLOT(completion_matches_callback()));
	connect(main_window->actionDynamicHelp, SIGNAL(triggered()), terminal, SLOT(dynamic_help_callback()));
	connect(main_window->actionStopProcess, SIGNAL(triggered()), terminal, SLOT(stop_process_callback()));
	connect(main_window->actionClearTerminal, SIGNAL(triggered()), terminal, SLOT(clear_callback()));

	//connect(oc, SIGNAL(widget_activated(BaseWidget *)), this, SLOT(widget_activated(BaseWidget *)));
	//oc->setOctavePath("/usr/bin/");
	oc->setOctavePath("");
	terminal->setOctaveConnection(oc);
	oc->startOctave();
	
	operations=new Operations(this,&active_widget,main_window);
	operations->setOctaveConnection(oc);
	
	//Se construyen los mens de los ficheros
	
	GenerateMenu generate_menu(main_window, operations);
	generate_menu.setPath( CONFIG_PATH "/menus");
	generate_menu.load_menu();
	
	QString home=QDir::home().path()+"/.qtoctave/";
	
	generate_menu.setPath( QString(home+"menus") );
	generate_menu.load_menu();
	
	//generate_menu.setPath("./menus");
	//generate_menu.load_menu();
	
	main_window->show_config_help_menus();
	
	connect(main_window->actionOctave_help, SIGNAL(triggered()), this, SLOT(help_octave()));
	connect(main_window->actions.value("qtoctave_help"),  SIGNAL(triggered()), this, SLOT(help_qtoctave()));
	connect(main_window->actionTable, SIGNAL(triggered()), this, SLOT(table()));
	connect(main_window->actionOpen, SIGNAL(triggered()), this, SLOT(open()));
	
	if ( modeWorkArea == "workspace" || modeWorkArea == "sdi")
	{
		connect(main_window->actionVariableList, SIGNAL(triggered()), this, SLOT(variable_list()));
	}
	else if (modeWorkArea == "dock")
	{
		//connect the variables list
		variable_list();
		connect(main_window->actionVariableList, SIGNAL(triggered()), this, SLOT(setVisibleVarList()));
	}
	
	//connect the Navigator
	connect(main_window->actionNavigator, SIGNAL(triggered()), this, 
					SLOT(setVisibleNavigator()));


	connect(main_window->actions.value("windows_layout"),  SIGNAL(triggered()), this, SLOT(windows_layout()));
	connect(main_window->actionRunFile, SIGNAL(triggered()),
		       				this, SLOT(run_file()));
	
	main_window->show();
}

void Main::open()
{
	Main *m;
	m=new Main;
}

void Main::widget_activated(BaseWidget *w)
{
	active_widget=w;
}

void Main::help_octave()
{
	Help *help=new Help(work_space);
	connect(help, SIGNAL(widget_activated(BaseWidget *)), this, SLOT(widget_activated(BaseWidget *)));
	if(work_space!=NULL) work_space->addWindow(help);
	if(get_config("help_path").isEmpty()) help->setSource( HELP_PATH );
	else help->setSource(get_config("help_path"));
	help->show();
}

void Main::help_qtoctave()
{
	Help *help=new Help(work_space);
	connect(help, SIGNAL(widget_activated(BaseWidget *)), this, SLOT(widget_activated(BaseWidget *)));
	if(work_space!=NULL) work_space->addWindow(help);
	if(get_config("qtoctave_help_path").isEmpty()) help->setSource( QTOCTAVE_HELP_PATH );
	else help->setSource(get_config("qtoctave_help_path"));
	help->setWindowTitle("QtOctave Help");
	help->show();
}

void Main::table(QString text)
{
	bool ok=true;
	if(text.isEmpty())
	{
		text = QInputDialog::getText(main_window, tr("Select table"),
			tr("Matrix name:"), QLineEdit::Normal, "", &ok);
	}
	if (ok && !text.isEmpty())
	{
		Table *table=new Table(work_space);
		connect(table, SIGNAL(widget_activated(BaseWidget *)), this, SLOT(widget_activated(BaseWidget *)));
		OctaveConnection *oc=terminal->getOctaveConnection();
		//connect(work_space, SIGNAL(windowActivated(QWidget *)), table, SLOT(windowActivated(QWidget *)));
		table->setOctaveConnection(oc);
		table->setMatrix(text);
		table->setWindowTitle("Table: "+text);
		if(work_space!=NULL) work_space->addWindow(table);
		table->show();
		table->windowActivated();
	}
}

void Main::run_file()
{
  QFileDialog openDialog(NULL, tr("Open"), ".");
  QStringList filters;

  filters << "Octave (*.m; *.M)";

  openDialog.setFilters(filters);
  openDialog.setAcceptMode(QFileDialog::AcceptOpen);
  openDialog.setDefaultSuffix("m");

  if(openDialog.exec() == QDialog::Accepted)
  {
    QFileInfo fileInfo(openDialog.selectedFiles().first());
    QString cmd;
    OctaveConnection *oc = terminal->getOctaveConnection();

    // Cambiar directorio
    cmd = QString("cd ") + fileInfo.dir().absolutePath();
    oc->command_enter(cmd);

    // Ejecutar
    cmd = fileInfo.baseName();
    oc->command_enter(cmd);
  }
}

void Main::variable_list()
{
	variableList=new VariableList(work_space);
	OctaveConnection *oc=terminal->getOctaveConnection();
	variableList->setOctaveConnection(oc);
	
	connect(variableList, SIGNAL(open_table(QString)), this, SLOT(table(QString)) );

	variableList->send_whos_command_to_octave();
	
	QString modeWorkArea = get_config("mode_work_area");	
	if ( modeWorkArea == "workspace")
	{
		work_space->addWindow(variableList);
		variableList->show();
	}
	else if (modeWorkArea == "dock")
	{
		//add dock from variables list 
		main_window->dockListVar->setWidget(variableList);
	}
	else if (modeWorkArea == "sdi")
	{
		variableList->show();
	}

}

void Main::windows_layout()
{
	if(work_space==NULL)
	{
		QMessageBox::information(main_window, tr("Information"), tr("Windows' Layout is only available in Workspace mode"), tr("Ok"));
		return;
	}
	WindowsLayout *wl=new WindowsLayout();
	wl->set_window_list( work_space->windowList() );
	wl->set_work_space( work_space );
	wl->show();
}

/** set visible or not the dock of var list*/
void Main::setVisibleVarList()
{
	main_window->dockListVar->
		setVisible(!main_window->dockListVar->isVisible());
}
/*this function show or not the Navigator dock*/
void Main::setVisibleNavigator()
{
	QString modeWorkArea = get_config("mode_work_area");	
	if ( modeWorkArea == "workspace" )
	{
		Navigator *nav=new Navigator(work_space);
		nav->setOctaveConnection(terminal->getOctaveConnection());
		work_space->addWindow(nav);
		nav->show();
	}
	else if (modeWorkArea == "dock" )
	{
		main_window->dockNavigator->
		setVisible(!main_window->dockNavigator->isVisible());
	}
	else if ( modeWorkArea == "sdi" )
	{
		Navigator *nav=new Navigator(work_space);
		nav->setOctaveConnection(terminal->getOctaveConnection());
		nav->show();
	}
	
}

QWidget *Main::mainWindowWidget()
{
	return main_window;
}

#include <QSplashScreen>

int main(int argn, char *argv[])
{
	QApplication a(argn,argv);
	//Se inicializa la configuraciÃ³n
	get_config("");
	//Show splash screen
	QSplashScreen *splash = new QSplashScreen( QPixmap(QString(ICON_PATH) + "/splash.png"),  Qt::WindowStaysOnTopHint );
	splash->showMessage ("Starting QtOctave",Qt::AlignBottom|Qt::AlignLeft);
        splash->show();
	Main m;
	splash->finish(m.mainWindowWidget());
	//Destroy splash screnn
	delete splash;
	return a.exec();
}


